/*
 *         
 * SmoothAlgorithms.mqh
 *   (): MetaTrader\\MQL5\Include
 */
//+------------------------------------------------------------------+ 
//|                                           ColorNonLagDotMACD.mq5 | 
//|                               Copyright  2011, Nikolay Kositsin | 
//|                              Khabarovsk,   farria@mail.redcom.ru | 
//+------------------------------------------------------------------+ 
#property copyright "Copyright  2011, Nikolay Kositsin"
#property link "farria@mail.redcom.ru" 
//----   
#property version   "1.00"
//----     
#property indicator_separate_window 
//----    4
#property indicator_buffers 4 
//----     
#property indicator_plots   2
//+-----------------------------------+
//|                 |
//+-----------------------------------+
#define RESET  0 //        
//+-----------------------------------+
//|       |
//+-----------------------------------+
//----      
#property indicator_type1 DRAW_COLOR_HISTOGRAM
//----      
#property indicator_color1 Gray,Lime,Blue,Red,Magenta
//----   - 
#property indicator_style1 STYLE_SOLID
//----     2
#property indicator_width1 2
//----   
#property indicator_label1 "NonLagDotMACD"

//----      
#property indicator_type2 DRAW_COLOR_LINE
//----      
#property indicator_color2 Gray,DodgerBlue,DarkOrange
//----   -  
#property indicator_style2 STYLE_DASHDOTDOT
//----     3
#property indicator_width2 3
//----    
#property indicator_label2  "Signal Line"
//+-----------------------------------+
//|         |
//+-----------------------------------+
input ENUM_APPLIED_PRICE Price=PRICE_CLOSE;   //  
input int                Filter= 0;
input double             Deviation=0;         // 
//----
input ENUM_MA_METHOD     Fast_Type=MODE_SMA;  //  
input int                Fast_Length_=12;     //   
//----
input ENUM_MA_METHOD     Slow_Type=MODE_SMA;  //  
input int                Slow_Length_=26;     //   
//----
input int Signal_MA=9; //   
input ENUM_MA_METHOD Signal_Method_=MODE_EMA; //  
//+-----------------------------------+

//----      
int Fast_Handle,Slow_Handle;
//----      
int start,macd_start=0;
//----   ,    
//      
double MACDBuffer[],SignBuffer[],ColorMACDBuffer[],ColorSignBuffer[];
//+------------------------------------------------------------------+
//   iPriceSeries                                     |
//   Moving_Average                                    | 
//+------------------------------------------------------------------+ 
#include <SmoothAlgorithms.mqh> 
//+------------------------------------------------------------------+    
//| MACD indicator initialization function                           | 
//+------------------------------------------------------------------+  
void OnInit()
  {
//----     
   macd_start=6*MathMax(Fast_Length_,Slow_Length_);
   start=macd_start+Signal_MA+1;
   
//----    Fast NonLagDot
   Fast_Handle=iCustom(NULL,0,"NonLagDot",Price,Fast_Type,Fast_Length_,Filter,Deviation,0);
   if(Fast_Handle==INVALID_HANDLE) Print("      Fast NonLagDot");
   
//----    Slow NonLagDot
   Slow_Handle=iCustom(NULL,0,"NonLagDot",Price,Slow_Type,Slow_Length_,Filter,Deviation,0);
   if(Slow_Handle==INVALID_HANDLE) Print("      Slow NonLagDot");

//----    MACDBuffer   
   SetIndexBuffer(0,MACDBuffer,INDICATOR_DATA);
//----      
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,macd_start);
//---      DataWindow
   PlotIndexSetString(0,PLOT_LABEL,"NonLagDotMACD");
//----   ,      
   PlotIndexSetDouble(0,PLOT_EMPTY_VALUE,EMPTY_VALUE);

//----     ,     
   SetIndexBuffer(1,ColorMACDBuffer,INDICATOR_COLOR_INDEX);
//----      
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,macd_start+1);

//----    SignBuffer   
   SetIndexBuffer(2,SignBuffer,INDICATOR_DATA);
//----      
   PlotIndexSetInteger(2,PLOT_DRAW_BEGIN,start);
//---      DataWindow
   PlotIndexSetString(2,PLOT_LABEL,"Signal SMA");
//----   ,      
   PlotIndexSetDouble(2,PLOT_EMPTY_VALUE,EMPTY_VALUE);

//----     ,     
   SetIndexBuffer(3,ColorSignBuffer,INDICATOR_COLOR_INDEX);
//----      
   PlotIndexSetInteger(3,PLOT_DRAW_BEGIN,start+1);

//----      
   string shortname;
   StringConcatenate(shortname,"NonLagDotMACD( ",Fast_Length_,", ",Slow_Length_,", ",Signal_MA," )");
//---           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits+1);
//----  
  }
//+------------------------------------------------------------------+  
//| MACD iteration function                                          | 
//+------------------------------------------------------------------+  
int OnCalculate(
                const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]
                )
  {
//----       
   if(BarsCalculated(Fast_Handle)<rates_total
    || BarsCalculated(Slow_Handle)<rates_total
    || rates_total<start) return(RESET);

//----   
   int first1,first2,first3,limit,bar;
//----       
   double Fast[],Slow[];
   
//----         
   ArraySetAsSeries(Fast,true);
   ArraySetAsSeries(Slow,true);

//----     OnCalculate()
   if(prev_calculated>rates_total || prev_calculated<=0)//      
     {
      first1=0; //        
      first2=macd_start+1; //        
      first3=start+1; //        
      limit=rates_total-macd_start; //      
     }
   else //      
     {
      first1=prev_calculated-1;
      first2=first1;
      first3=first1;
      limit=rates_total-prev_calculated; //      
     }
     
//----      
   if(CopyBuffer(Fast_Handle,0,0,limit+1,Fast)<=0) return(RESET);
   if(CopyBuffer(Slow_Handle,0,0,limit+1,Slow)<=0) return(RESET);

//----       
   ArraySetAsSeries(MACDBuffer,true);

//----    
   for(bar=limit; bar>=0 && !IsStopped(); bar--) MACDBuffer[bar]=Fast[bar]-Slow[bar];     

//----        
   ArraySetAsSeries(MACDBuffer,false);    
   
//----    CMoving_Average   MASeries_Cls.mqh
   static CMoving_Average MA; 
     
   for(bar=first1; bar<rates_total && !IsStopped(); bar++)
      SignBuffer[bar]=MA.MASeries(macd_start,prev_calculated,rates_total,Signal_MA,Signal_Method_,MACDBuffer[bar],bar,false);

//----     MACD
   for(bar=first2; bar<rates_total && !IsStopped(); bar++)
     {
      ColorMACDBuffer[bar]=0;

      if(MACDBuffer[bar]>0)
        {
         if(MACDBuffer[bar]>MACDBuffer[bar-1]) ColorMACDBuffer[bar]=1;
         if(MACDBuffer[bar]<MACDBuffer[bar-1]) ColorMACDBuffer[bar]=2;
        }

      if(MACDBuffer[bar]<0)
        {
         if(MACDBuffer[bar]<MACDBuffer[bar-1]) ColorMACDBuffer[bar]=3;
         if(MACDBuffer[bar]>MACDBuffer[bar-1]) ColorMACDBuffer[bar]=4;
        }
     }

//----     
   for(bar=first3; bar<rates_total && !IsStopped(); bar++)
     {
      ColorSignBuffer[bar]=0;
      if(MACDBuffer[bar]>SignBuffer[bar-1]) ColorSignBuffer[bar]=1;
      if(MACDBuffer[bar]<SignBuffer[bar-1]) ColorSignBuffer[bar]=2;
     }
//----     
   return(rates_total);
  }
//+------------------------------------------------------------------+
